"""
.. module:: mdns
    :platform: Darwin, Linux, Unix, Windows
    :synopsis: The mdns package contains modules for engaging in discovery operations
               with mdns services.

.. moduleauthor:: Myron Walker <myron.walker@gmail.com>
"""

__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []
__version__ = "1.0.0"
__maintainer__ = "Myron Walker"
__email__ = "myron.walker@gmail.com"
__status__ = "Development" # Prototype, Development or Production
__license__ = "MIT"



"""
.. module:: dnspointer
    :platform: Darwin, Linux, Unix, Windows
    :synopsis: Contains the DnsService object which represents a DNS service record.

.. moduleauthor:: Myron Walker <myron.walker@gmail.com>
"""

__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []
__version__ = "1.0.0"
__maintainer__ = "Myron Walker"
__email__ = "myron.walker@gmail.com"
__status__ = "Development" # Prototype, Development or Production
__license__ = "MIT"

from typing import Union, TYPE_CHECKING

import threading

from akit.interop.dns.dnsrecord import DnsRecord

if TYPE_CHECKING:
    from akit.interop.dns.dnspacketwriter import DnsPacketWriter

class DnsService(DnsRecord):
    """
        A DNS service record
    """

    def __init__(self, name: str, rtype: int, rclass: int, ttl: Union[float, int], priority: int,
                 weight: int, port: int, server: str ) -> None:
        DnsRecord.__init__(self, name, rtype, rclass, ttl)
        self._priority = priority
        self._weight = weight
        self._port = port
        self._server = server

        self._thread_response = None
        self._running = False
        self._shutdown_gate = threading.Event()
        self._shutdown_gate.set()
        return

    @property
    def port(self):
        return self._port

    @property
    def priority(self):
        return self._priority

    @property
    def server(self):
        return self._server

    @property
    def weight(self):
        return self._weight
    
    def write(self, out: 'DnsPacketWriter') -> None:
        """
            Used in constructing an outgoing packet
        """
        out.write_short(self._priority)
        out.write_short(self._weight)
        out.write_short(self._port)
        out.write_name(self._server)
        return

    def __eq__(self, other: DnsRecord) -> bool:
        """
            Tests equality on priority, weight, port and server
        """
        # Call DnsRecord equality operator first because it can eliminate alot of equality checks early
        iseq = False
        if DnsRecord.__eq__(self, other):
            iseq = self._priority == other._priority and self._weight == other._weight and self._port == other._port and self._server == other._server
        return iseq

    def __ne__(self, other: DnsRecord) -> bool:
        """
            Non-equality test
        """
        return not self.__eq__(other)

    def __str__(self) -> str:
        """
            String representation
        """
        other = self._server + " " + self._port+ " " + self._priority + " " + self._weight
        strval = self.as_dns_string(other=other)
        return strval




from abc import ABC, abstractmethod

from rich.progress import Progress

from akit.exceptions import AKitNotOverloadedError

class ProcedureContext(ABC):

    def __init__(self, identifier: str):
        self._identifier = identifier
        self._progress_console = None
        return

    @property
    def identifier(self):
        return self._identifier

    @property
    def progress_console(self):
        return self._progress_console

    @abstractmethod
    def attach_to_progress_console(self, progress_console: Progress):
        errmsg = "The ProcedureContext:attach_to_progress_console method must be overloaded by derived types."
        raise AKitNotOverloadedError(errmsg)




import traceback

def format_exc_lines():
    """
        Gets a 'format_exc' result and splits it into mutliple lines.
    """
    rtn_lines = traceback.format_exc().splitlines()
    return rtn_lines



import os

from akit.exceptions import AKitSemanticError
from akit.environment.variables import ActivationProfile, AKIT_VARIABLES, JOB_TYPES

__activation_profile__ = ActivationProfile.TestRun

# Guard against attemps to activate more than one, activation profile.
if AKIT_VARIABLES.AKIT_ACTIVATION_PROFILE is not None:
    errmsg = "An attempt was made to activate multiple environment activation profiles. profile={}".format(
        AKIT_VARIABLES.AKIT_ACTIVATION_PROFILE
    )
    raise AKitSemanticError(errmsg)

AKIT_VARIABLES.AKIT_ACTIVATION_PROFILE = ActivationProfile.TestRun

AKIT_VARIABLES.AKIT_JOBTYPE = JOB_TYPES.TESTRUN

os.environ["AKIT_JOBTYPE"] = AKIT_VARIABLES.AKIT_JOBTYPE

import akit.activation.base




from typing import NamedTuple

class StationInfo(NamedTuple):
    mac: str
    inactive_time: int
    rx_bytes: int
    rx_packets: int
    rx_drop_misc: int
    rx_bitrate: int
    tx_bytes: int
    tx_packets: int
    tx_retries: int
    tx_failed: int
    tx_bitrate: int
    signal: int
    signal_avg: int
    exp_throughput: int
    count_authorized: int
    count_authenticated: int
    count_associated: int
    wmm: int
    mfp: int
    tdls_peer: str
    dtim_period: str
    beacon_interval: float
    preamble: int
    short_preamble: int
    short_slot_time: int
    connected_time: int




class DiagnosticLabel:
    PRERUN_DIAGNOSTIC = "prerun-diagnostic"
    POSTRUN_DIAGNOSTIC = "postrun-diagnostic"

class RuntimeConfigPaths:
    DIAGNOSTIC_PRERUN = "/configuration/diagnostics/prerun-diagnostic"
    DIAGNOSTIC_POSTRUN = "/configuration/diagnostics/postrun-diagnostic"




TIMEDELTA_RENEWAL_WINDOW = 120

from akit.aspects import AspectsUPnP

UPNP_CALL_COMPLETION_TIMEOUT = 30
UPNP_CALL_COMPLETION_INTERVAL = 5

UPNP_CALL_INACTIVITY_TIMEOUT = 30
UPNP_CALL_INACTIVITY_INTERVAL = 1

DEFAULT_UPNP_CALL_ASPECTS = AspectsUPnP(
    completion_timeout=UPNP_CALL_COMPLETION_TIMEOUT,
    completion_interval=UPNP_CALL_COMPLETION_INTERVAL,
    inactivity_timeout=UPNP_CALL_INACTIVITY_TIMEOUT,
    inactivity_interval=UPNP_CALL_INACTIVITY_INTERVAL)